@@ -3,7 +3,7 @@
 ## Purpose:     
 ## Author:      Alex Thuering
 ## Created:     2005/01/19
-## RCS-ID:      $Id: interfaces.py,v 1.36 2013/10/15 19:30:41 ntalex Exp $
+## RCS-ID:      $Id: interfaces.py,v 1.40 2014/03/27 08:38:04 ntalex Exp $
 ## Copyright:   (c) 2005 Alex Thuering
 ## Notes:		some modules adapted from svgl project
 ##############################################################################
@@ -81,6 +81,8 @@ inter.include_methods.append('    void SkewY(double angle);\n')
 inter.include_methods.append('    void UpdateMatrix(wxSVGMatrix& matrix) const;\n')
 inter.include_methods.append('    static wxSVGTransformable* GetSVGTransformable(wxSVGElement& element);\n')
 inter.include_methods.append('    static const wxSVGTransformable* GetSVGTransformable(const wxSVGElement& element);\n')
+inter.include_methods_protected.append('    inline wxSVGAnimatedTransformList& GetTransformList() { return m_transform; }\n')
+inter.include_methods_protected.append('    friend class wxSVGAnimateTransformElement;\n')
 
 # SVGFitToViewBox
 inter = interface()
@@ -197,6 +199,7 @@ inter.user_defined_destructor=1
 # SVGAngle
 inter = interface()
 interfaces["SVGAngle"]=inter
+inter.include_attributes_init = [["valueInSpecifiedUnits", "0"]]
 inter.include_methods.append('    wxSVGAngle() : m_unitType(wxSVG_ANGLETYPE_UNKNOWN), m_value(0) {}\n')
 inter.include_methods.append('    wxSVGAngle(double v) : m_unitType(wxSVG_ANGLETYPE_UNSPECIFIED), m_value(v) {}\n')
 inter.include_methods.append('    virtual ~wxSVGAngle() {}\n')
@@ -217,6 +220,15 @@ inter.include_includes = ["String_wxsvg"]
 inter.user_defined_constructor=1
 inter.user_defined_destructor=1
 
+# SVGNumber
+inter = interface()
+interfaces["SVGNumber"]=inter
+inter.include_methods.append('    wxSVGNumber(): m_value(0) {}\n')
+inter.include_methods.append('    wxSVGNumber(double value): m_value(value) {}\n')
+inter.include_methods.append('    virtual ~wxSVGNumber() {}\n')
+inter.user_defined_constructor=1
+inter.user_defined_destructor=1
+
 # SVGPoint
 inter = interface()
 interfaces["SVGPoint"]=inter
@@ -283,7 +295,7 @@ inter.user_defined_destructor=1
 inter = interface()
 interfaces["SVGTransform"]=inter
 inter.include_get_set_attributes = [["double", "cx", False, False], ["double", "cy", False, False]]
-inter.include_methods.append('    wxSVGTransform(const wxSVGMatrix& matrix): m_type(wxSVG_TRANSFORM_MATRIX), m_matrix(matrix), m_angle(0) {}\n')
+inter.include_methods.append('    wxSVGTransform(const wxSVGMatrix& matrix): m_type(wxSVG_TRANSFORM_MATRIX), m_matrix(matrix), m_angle(0), m_cx(0), m_cy(0) {}\n')
 inter.include_methods.append('    virtual ~wxSVGTransform() {}\n')
 inter.include_methods.append('    \n')
 inter.include_methods.append('    wxString GetValueAsString() const;\n')
@@ -343,7 +355,7 @@ for name in ["SVGLineElement", "SVGPolylineElement", "SVGPolygonElement",
         inter.include_attributes.append('  public:\n')
         inter.include_attributes.append('    inline wxSVGCanvasItem* GetCanvasItem() { return m_canvasItem; }\n')
         inter.include_attributes.append('    void SetCanvasItem(wxSVGCanvasItem* canvasItem);\n\n')
-        inter.include_attributes_init = [["canvasItem", True]]
+        inter.include_attributes_init = [["canvasItem", "NULL"]]
         inter.include_fwd_decls = ["SVGCanvasItem"]
     inter.include_methods.append('    wxSVGRect GetBBox(wxSVG_COORDINATES coordinates = wxSVG_COORDINATES_USER);\n')
     inter.include_methods.append('    wxSVGRect GetResultBBox(wxSVG_COORDINATES coordinates = wxSVG_COORDINATES_USER);\n')
@@ -391,12 +403,24 @@ inter.include_methods.append('''
     double GetQualifiedFx() const;
     double GetQualifiedFy() const;\n
 ''')
-# SVGRadialGradientElement
 
+# SVGAnimationElement
 inter = interface()
 interfaces["SVGAnimationElement"]=inter
 inter.include_methods.append('''
-    void ApplyAnimation();\n
+    virtual void ApplyAnimation();\n
+''')
+
+# SVGAnimateTransformElement
+inter = interface()
+interfaces["SVGAnimateTransformElement"]=inter
+inter.include_attributes.append('''  protected:
+    int m_transformIdx;
+
+''')
+inter.include_attributes_init = [["transformIdx", "-1"]]
+inter.include_methods.append('''
+    virtual void ApplyAnimation();\n
 ''')
 
 # SVGDocument
@@ -453,6 +477,6 @@ inter.include_methods.append('''    wxSVGDocument() { Init(); }
 ''')
 inter.include_fwd_decls = ["SVGSVGElement","SVGElement","SVGCanvas","ProgressDialog"]
 inter.include_includes = ["SVGRect","SVGMatrix","<wx/image.h>"]
-inter.user_defined_contructor=1
+inter.user_defined_constructor=1
 inter.user_defined_destructor=1
 
